package com.ruoyi.market.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.utils.idgen.IdGenerate;
import com.ruoyi.market.domain.MarketAwardLibrary;
import com.ruoyi.market.domain.MarketDrawActivity;
import com.ruoyi.market.domain.MarketDrawActivityProbability;
import com.ruoyi.market.domain.MarketDrawTemplate;
import com.ruoyi.market.domain.vo.DrawActivityQueryVo;
import com.ruoyi.market.domain.vo.DrawActivitySaveVo;
import com.ruoyi.market.domain.vo.DrawTemplateQueryVo;
import com.ruoyi.market.mapper.MarketDrawActivityMapper;
import com.ruoyi.market.mapper.MarketDrawTemplateMapper;
import com.ruoyi.market.service.MarketAwardLibraryService;
import com.ruoyi.market.service.MarketDrawActivityProbabilityService;
import com.ruoyi.market.service.MarketDrawActivityService;
import com.ruoyi.market.service.MarketDrawTemplateService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Objects;

@Service
public class MarketDrawActivityServiceImpl extends ServiceImpl<MarketDrawActivityMapper, MarketDrawActivity>
        implements MarketDrawActivityService {

    @Autowired
    MarketDrawTemplateService marketDrawTemplateService;

    @Autowired
    MarketAwardLibraryService marketAwardLibraryService;

    @Autowired
    MarketDrawActivityProbabilityService marketDrawActivityProbabilityService;

    /**
     * 获取抽奖活动列表
     * @param page
     * @param drawActivityQueryVo
     * @return
     */
    @Override
    public Page<MarketDrawActivity> getDrawActivityPage(Page page, DrawActivityQueryVo drawActivityQueryVo) {
        LambdaQueryWrapper<MarketDrawActivity> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(!Objects.isNull(drawActivityQueryVo.getDrawName()), MarketDrawActivity::getName, drawActivityQueryVo.getDrawName());
        return this.page(page, queryWrapper);
    }

    /**
     * 添加抽奖活动
     * @param drawActivitySaveVo
     * @return
     */
    @Transactional
    @Override
    public boolean addDrawActivity(DrawActivitySaveVo drawActivitySaveVo) {
        String drawId = IdGenerate.nextId();
        //验证抽奖模板
        MarketDrawTemplate template = marketDrawTemplateService.getById(drawActivitySaveVo.getDrawActivity().getTemplateId());
        if(Objects.isNull(template)){
            throw new RuntimeException("抽奖模板不存在");
        }

        //验证奖品
        drawActivitySaveVo.getAwardProbabilityList().forEach(item->{
            MarketAwardLibrary award = marketAwardLibraryService.getById(item.getId());
            if(!Objects.isNull(award)){
                throw new RuntimeException("奖品不存在");
            }
        });

        drawActivitySaveVo.getDrawActivity().setId(drawId);

        drawActivitySaveVo.getAwardProbabilityList().forEach(item->{
            item.setId(IdGenerate.nextId());
            item.setDrawId(drawId);
        });

        marketDrawActivityProbabilityService.saveBatch(drawActivitySaveVo.getAwardProbabilityList());
        this.save(drawActivitySaveVo.getDrawActivity());
        return true;
    }

    @Override
    public MarketDrawActivity getDrawActivityById(String drawActivityId) {
        MarketDrawActivity drawActivity = this.getById(drawActivityId);
        List<MarketDrawActivityProbability> probabilitysByDrawActivityId = marketDrawActivityProbabilityService.getProbabilitysByDrawActivityId(drawActivityId);
        drawActivity.setProbabilitys(probabilitysByDrawActivityId);
        return drawActivity;
    }
}
